import torch
import torch.nn as nn
from torchvision import transforms, models
from PIL import Image
import sys
from torch.ao.quantization import quantize_dynamic

def load_model(model_path, num_classes):
    # モデル構築（MobileNetV2）
    model = models.mobilenet_v2()
    model.classifier[1] = nn.Linear(model.last_channel, num_classes)
    
    # 量子化（同様の設定で）
    model = quantize_dynamic(
        model,
        {nn.Linear},
        dtype=torch.qint8
    )
    
    model.load_state_dict(torch.load(model_path, map_location='cpu'))
    model.eval()
    return model

def predict(image_path, model, class_names):
    # 前処理
    transform = transforms.Compose([
        transforms.Resize((224, 224)),
        transforms.ToTensor(),
        transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
    ])

    image = Image.open(image_path).convert('RGB')
    input_tensor = transform(image).unsqueeze(0)  # バッチ次元を追加

    with torch.no_grad():
        outputs = model(input_tensor)
        _, predicted = torch.max(outputs, 1)
    
    predicted_class = class_names[predicted.item()]
    print(f"予測結果: {predicted_class}")

if __name__ == "__main__":
    # --- 設定 ---
    image_path = "test_image.jpg"  # ← ここに判定したい画像ファイルのパスを指定
    model_path = "mobilenetv2_quantized.pth"
    
    # 学習に使用したクラス名（trainディレクトリのサブフォルダ名の順）
    class_names = ['large_hub', 'minifig']  # ← あなたのデータセットに合わせて修正

    num_classes = len(class_names)
    
    model = load_model(model_path, num_classes)
    predict(image_path, model, class_names)
